import numpy as np
from ase.build import molecule
from ase.visualize import view
from gpaw import GPAW
from gpaw.poisson_extended import ExtendedPoissonSolver as EPS
from gpaw.utilities import h2gpts

atoms = molecule('NH3')
atoms.numbers[0] = 8
atoms.center(vacuum=3.0)
#atoms.center(vacuum=1.5, axis=2)

#atoms.cell[2, 2] += 256 / 5.

class EPS2(EPS):
    def __init__(self, direction, *args, **kwargs):
        super(EPS2, self).__init__(*args, **kwargs)
        assert direction == 2
        self.direction = direction


    def set_grid_descriptor(self, gd):
        super(EPS2, self).set_grid_descriptor(gd)
        cell_cv = self.gd.cell_cv
        for i in range(2):
            dir2 = (self.direction + 1 + i) % 3
            assert np.dot(cell_cv[self.direction], cell_cv[dir2]) < 1e-8
        assert not self.gd.pbc_c[self.direction]

    def _solve(self, phi, rho, charge=None, eps=None, maxcharge=1e-6,
               zero_initial_phi=False):

        # XXX remember to possibly fix off-by-one error

        from gpaw.utilities.grid_redistribute import AlignedGridRedistributor
        print(charge)
        print(self.gd.integrate(rho))

        gd = self.gd
        redist = AlignedGridRedistributor(gd, 0, 2)
        rho_xy = redist.forth(rho)

        oldcellstart = gd.extend_N_cd[2, 0]
        oldcellstop = oldcellstart + self.gd_original.N_c[2]

        #offset_c = gd.extend_N_cd[:, 0]
        #oldcellstart = offset_c[2]
        #oldcellstop = gd.extend_N_cd[:, 1][2]
        #print('old', oldcellstart, oldcellstop)
        #print(gd.extend_N_cd)

        rho_central = rho_xy[:, :, oldcellstart:oldcellstop]
        #print('rho_xy', rho_xy.shape)
        rho_xy[:, :, :oldcellstart] -= rho_central[:, :, ::-1]
        mirror_rho = redist.back(rho_xy)


        rho = mirror_rho

        #pl.plot(rho.sum(0).sum(0))
        #pl.show()

        if 0:
            Nz_orig = self.gd_original.N_c[2]
            Nz = self.gd.N_c[2]
            assert (Nz - Nz_orig) % 2 == 0
            Nz_pad = (Nz - Nz_orig) // 2

            #startpoint = 0
            #endpoint = nz_pad
            #npoints = endpoint - startpoint

            r_vg = self.gd.get_grid_point_coordinates()
            z_g = r_vg[self.direction, 0, 0, :]  # XXX z-specific

            #w_g[

            bigLz = self.gd.cell_cv[2, 2]
            smallLz = self.gd_original.cell_cv[2, 2]
            zcut = (bigLz - smallLz) / 2.0 * 0.1  # 'safety' buffer?

            def chargefunc(z):
                if z < zcut:
                    return np.sin(np.pi * z / zcut)**2
                else:
                    return 0.0

            ourzcharge_g = np.array([chargefunc(z) for z in z_g])

            import pylab as pl
            #pl.plot(z_g, ourzcharge_g)
            #pl.show()

            ourcharge_g = self.gd.zeros()
            ourcharge_g[:, :, :] = ourzcharge_g[None, None, :]
            totalcharge = self.gd.integrate(ourcharge_g, global_integral=True)
            ourcharge_g *= -(charge / totalcharge)

            rho += ourcharge_g

        x = super(EPS2, self)._solve(phi, rho, charge=None, eps=None,
                                     maxcharge=1e-6,
                                     zero_initial_phi=False)

        import pylab as pl
        pl.plot(rho.sum(0).sum(0))
        #pl.plot(mirror_rho.sum(0).sum(0), ls='--')

        #pl.plot(z_g, rho.sum(0).sum(0), label='rho', ls='--',
        #        color='b')
        pl.plot(phi.sum(0).sum(0), label='phi', ls='--')
        #pl.legend()
        pl.show()
        return x

#atoms.pbc = 1
gpts = h2gpts(0.1, atoms.cell, idiv=16)
extgpts = (gpts[0], gpts[1], gpts[2] * 3)

calc = GPAW(mode='lcao',
            gpts=gpts,
            charge=1,
            basis='dzp',
            poissonsolver=EPS2(2,
                               extended={'gpts': extgpts, 'useprev': True}),
            txt='gpaw.txt')

atoms.calc = calc
atoms.get_potential_energy()
calc.write('out.gpw')

#atoms.cell[2, 2] += 20.0
#view(atoms)
